//-------------------------------------------------------------------------------------------------+
// GlobalMapViewer.h
// This RTC is used to show Global Map
//-------------------------------------------------------------------------------------------------+

#ifndef GlobalMapViewer_H
#define GlobalMapViewer_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service Consumer stub headers
#include "MRFCStub.h"
#include "AislGlobalPlanningStub.h"
#include "IIS2Stub.h"

#include "cv.h"
#include "highgui.h"

#include "MapToImage.h"

using namespace RTC;
using namespace IIS;


//-------------------------------------------------------------------------------------------------+
// GlobalMapViewer
//-------------------------------------------------------------------------------------------------+
class GlobalMapViewer : public RTC::DataFlowComponentBase{
public:
	GlobalMapViewer(RTC::Manager* manager);
	~GlobalMapViewer();
	virtual RTC::ReturnCode_t onInitialize();
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);
	virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

protected:
	int m_color_reverse;
	int m_map_type;
	double m_view_scale;

	IIS::TimedPose2D m_robot_pose;
	InPort<IIS::TimedPose2D> m_robot_poseIn;
	IIS::TimedPose2DSeq m_poses;
	InPort<IIS::TimedPose2DSeq> m_posesIn;
	IIS::TimedPoseVel2DSeq m_path;
	InPort<IIS::TimedPoseVel2DSeq> m_pathIn;
	TUT::TimedPersonsGlobalPosition m_persons;
	InPort<TUT::TimedPersonsGlobalPosition> m_personsIn;
  
	// CORBA Port declaration
	RTC::CorbaPort m_AbsoluteMapServicePort;
	RTC::CorbaPort m_AreaInfoServicePort;

	// Consumer declaration
	RTC::CorbaConsumer<MRFC::AbsoluteMapService> m_AbsoluteMapService;
	RTC::CorbaConsumer<TUT::AreaInfoService> m_AreaInfoService;


private:
	bool input_robot, input_poses, input_path, input_persons;
	MapToImage *convert;

	void DrowMap();
	void InputRobot();
	void InputPoses();
	void InputPath();
	void InputPersons();
	bool CheckConnectServicePort();
	bool CheckConnectAreaInfoService();
};



//-------------------------------------------------------------------------------------------------+
extern "C"
{
  DLL_EXPORT void GlobalMapViewerInit(RTC::Manager* manager);
};

#endif // GlobalMapViewer_H
