//-------------------------------------------------------------------------------------------------+
// MapToImage.h
// Define functions of MapToImage class that converts global map data to image
//-------------------------------------------------------------------------------------------------+

#include "MapToImage.h"
#include <vector>

using namespace std;


//------------------------------------------------------------------------------------------------+
// Define of macro
//------------------------------------------------------------------------------------------------+
#define Iblue(i,x,y) ((uchar*)((i)->imageData + (i)->widthStep*(y)))[3*(x)]		// 24rbgJ[A
#define Igreen(i,x,y) ((uchar*)((i)->imageData + (i)->widthStep*(y)))[3*(x)+1]	// 24rbgJ[A
#define Ired(i,x,y) ((uchar*)((i)->imageData + (i)->widthStep*(y)))[3*(x)+2]	// 24rbgJ[A


//-------------------------------------------------------------------------------------------------+
// Floatn}̕`
//-------------------------------------------------------------------------------------------------+
void MapToImage::DrawMap( MRFC::TimedFloatAbsoluteOGMapData *mapdata, bool color_reverse, double scale ){

	if( !mapdata || mapdata->cells.length() <= 0 || scale <= 0.0 ) return;

	x_scale = mapdata->mapconfig.xScale / scale;
	y_scale = mapdata->mapconfig.yScale / scale;

	origin_x = mapdata->mapconfig.origin.position.x;
	origin_y = mapdata->mapconfig.origin.position.y;

	// 摜𐶐
	IplImage *map_image = cvCreateImage( cvSize( mapdata->mapconfig.width, mapdata->mapconfig.height ), IPL_DEPTH_8U, 3 );

	UpdateAnimation();

	DrawFloatMap( map_image, mapdata, color_reverse );

	if( image ) cvReleaseImage( &image );
	image = cvCreateImage( cvSize( (int)( scale * mapdata->mapconfig.width ), (int)( scale * mapdata->mapconfig.height )), IPL_DEPTH_8U, 3 );

	cvResize( map_image, image, CV_INTER_AREA );
	cvReleaseImage( &map_image );
}


//-------------------------------------------------------------------------------------------------+
// Octetn}̕`
//-------------------------------------------------------------------------------------------------+
void MapToImage::DrawMap( MRFC::TimedAbsoluteOGMapData *mapdata, bool color_reverse, double scale ){

	if( !mapdata || mapdata->cells.length() <= 0 || scale <= 0.0 ) return;

	x_scale = mapdata->mapconfig.xScale / scale;
	y_scale = mapdata->mapconfig.yScale / scale;

	origin_x = mapdata->mapconfig.origin.position.x;
	origin_y = mapdata->mapconfig.origin.position.y;

	// 摜𐶐
	IplImage *map_image = cvCreateImage( cvSize( mapdata->mapconfig.width, mapdata->mapconfig.height ), IPL_DEPTH_8U, 3 );

	UpdateAnimation();

	DrawOctetMap( map_image, mapdata, color_reverse );

	if( image ) cvReleaseImage( &image );
	image = cvCreateImage( cvSize( (int)( scale * mapdata->mapconfig.width ), (int)( scale * mapdata->mapconfig.height )), IPL_DEPTH_8U, 3 );

	cvResize( map_image, image, CV_INTER_AREA );
	cvReleaseImage( &map_image );
}


//------------------------------------------------------------------------------------------------+
// {bg̕`
//------------------------------------------------------------------------------------------------+
void MapToImage::DrawRobot( RTC::Pose2D pose ){

	CvScalar colour = CV_RGB( 255, 127, 0 );

	Circle( image, pose.position.x, pose.position.y, for_anime, colour );
	Segment( image, pose.position.x, pose.position.y, pose.heading, 8.0, colour );
}


//------------------------------------------------------------------------------------------------+
// ʒupn̕`
//------------------------------------------------------------------------------------------------+
void MapToImage::DrawPoses( IIS::TimedPose2DSeq poses ){

	CvScalar colour = CV_RGB( 255, 0, 0 );

	for( int i=0; i<(int)poses.data.length(); i++ ){

		Circle( image, poses.data[i].position.x, poses.data[i].position.y, for_anime, colour );
		Segment( image, poses.data[i].position.x, poses.data[i].position.y, poses.data[i].heading, 8.0, colour );
	}
}


//------------------------------------------------------------------------------------------------+
// oH̕`
//------------------------------------------------------------------------------------------------+
void MapToImage::DrawPath( IIS::TimedPoseVel2DSeq path ){

	// ԖړInԐ\
	for( int i=1; i<(int)path.data.length(); i++ ){

		ConnectLine( image, path.data[i-1].pose.position.x, path.data[i-1].pose.position.y, path.data[i].pose.position.x, path.data[i].pose.position.y, CV_RGB( 255, 0, 255 ));
	}

	CvScalar colour = CV_RGB( 0, 127, 255 );

	// ԖړIn\
	for( int i=0; i<(int)path.data.length(); i++ ){

		int radius = ( i == (int)path.data.length() -1 )? for_anime: 0;

		Circle( image, path.data[i].pose.position.x, path.data[i].pose.position.y, radius, colour );
		Segment( image, path.data[i].pose.position.x, path.data[i].pose.position.y, path.data[i].pose.heading, 8.0, colour );
	}
}


//------------------------------------------------------------------------------------------------+
// l̕`
//------------------------------------------------------------------------------------------------+
void MapToImage::DrawPersons( TUT::TimedPersonsGlobalPosition persons ){

	for( int i=0; i<(int)persons.data.length(); i++ ){

		Circle( image, persons.data[i].x, persons.data[i].y, for_anime, CV_RGB( 0, 192, 0 ));
	}
}


//------------------------------------------------------------------------------------------------+
// TuGA֊s̕`
//------------------------------------------------------------------------------------------------+
void MapToImage::DrawAreaInfo( TUT::TimedAreaInfo *areainfo ){

	for( int i=0; i< areainfo->data.length(); i++ ){

		vector<CvPoint> points;
		for( int v=0; v<areainfo->data[i].vertexes.length(); v++ ){

			int x = Convert( areainfo->data[i].vertexes[v].x );
			int y = Convert( areainfo->data[i].vertexes[v].y, image->height );

			points.push_back( cvPoint( x, y ) );
		}
		points.push_back( points.front() );

		for( int i=1; i<(int)points.size(); i++ ){

			cvLine( image, points[i-1], points[i], CV_RGB( 127, 0, 255 ) );
		}
	}
}


//-------------------------------------------------------------------------------------------------+
// Aj[Vpϐ̍XV
//-------------------------------------------------------------------------------------------------+
void MapToImage::UpdateAnimation(){

	if( 4 < for_anime ) for_anime = 0;
	else for_anime++;
}


//-------------------------------------------------------------------------------------------------+
// Floatn}̕`
//-------------------------------------------------------------------------------------------------+
void MapToImage::DrawFloatMap( IplImage *img, MRFC::TimedFloatAbsoluteOGMapData *mapdata, bool color_reverse ){

	for( int i=0; i<img->height; i++ ) for( int x=0; x<img->width; x++){

		int y = img->height - i - 1;

		if( mapdata->cells[x + i * img->width] < 0.0 ) Put( img, x, y, 100, 100, 255 );

		else if( 1.0 < (float)mapdata->cells[x + i * img->width] ) Put( img, x, y, 100, 100, 255 );

		else{
			int value = (int)( 255.0 * mapdata->cells[x + i * img->width] );

			if( color_reverse ) value = 255 - value;

			Put( img, x, y, value, value, value );
		}
	}
}


//-------------------------------------------------------------------------------------------------+
// Octetn}̕`
//-------------------------------------------------------------------------------------------------+
void MapToImage::DrawOctetMap( IplImage *img, MRFC::TimedAbsoluteOGMapData *mapdata, bool color_reverse ){

	for( int i=0; i<img->height; i++ ) for( int x=0; x<img->width; x++){

		int y = img->height - i - 1;

		if( mapdata->cells[x + i * img->width] < 0 ) Put( img, x, y, 100, 100, 255 );

		else if( 100 < (int)mapdata->cells[x + i * img->width] ) Put( img, x, y, 100, 100, 255 );

		else{
			int value = (int)( 2.55 * (double)mapdata->cells[x + i * img->width] );

			if( color_reverse ) value = 255 - value;

			Put( img, x, y, value, value, value );
		}
	}
}


//------------------------------------------------------------------------------------------------+
// wfwFɂ
//------------------------------------------------------------------------------------------------+
void MapToImage::Put( IplImage *img, int x, int y, int r, int g, int b ){

	if( !img || x < 0 || y < 0 || img->width <= x || img->height <= y ) return;

	Ired(   img, x, y ) = r;
	Igreen( img, x, y ) = g;
	Iblue(  img, x, y ) = b;
}


//------------------------------------------------------------------------------------------------+
// ~`
//------------------------------------------------------------------------------------------------+
void MapToImage::Circle( IplImage *img, double x, double y, int radius, CvScalar color ){

	if( !img ) return;

	CvPoint point = { Convert(x), Convert( y, img->height ) };

	cvCircle( img, point,          3, color, CV_FILLED );	// S
	cvCircle( img, point, radius + 3, color );				// 
}


//------------------------------------------------------------------------------------------------+
// `
//------------------------------------------------------------------------------------------------+
void MapToImage::Segment( IplImage *img, double x, double y, double heading, double length, CvScalar color ){

	if( !img ) return;

	CvPoint from = { Convert(x), Convert( y, img->height ) };
	CvPoint to   = { from.x + (int)( length * cos( heading )), from.y - (int)( length * sin( heading )) };

	cvLine( img, from, to, color, 1, 8, 0 );
}


//------------------------------------------------------------------------------------------------+
// 2_Ԑ`
//------------------------------------------------------------------------------------------------+
void MapToImage::ConnectLine( IplImage *img, double x1, double y1, double x2, double y2, CvScalar color ){

	if( !img ) return;

	CvPoint from = { Convert(x1), Convert( y1, img->height ) };
	CvPoint to   = { Convert(x2), Convert( y2, img->height ) };

	cvLine( img, from, to, color, 1, 8, 0 );
}


//------------------------------------------------------------------------------------------------+
// W摜Wɕϊ
//------------------------------------------------------------------------------------------------+
int MapToImage::Convert( double x ){

	return (int)(( x - origin_x ) / x_scale );
}


//------------------------------------------------------------------------------------------------+
// W摜Wɕϊ
//------------------------------------------------------------------------------------------------+
int MapToImage::Convert( double y, int height ){

	return ( height - (int)(( y - origin_y ) / y_scale ));
}


